/***********************************************************************************
* T-SQL: Simple Changes That Go a Long Way
* Dave Valentine
* Non-Used Indexes Demo SQL Script
***********************************************************************************/

USE AdventureWorks2012;
GO
--SET STATISTICS IO ON so we can see disk activity
SET STATISTICS IO ON;
GO


/***********************************************************************************
* Create some usage statistics on Production.TransactionHisory
***********************************************************************************/
DECLARE @i int = 1;
DECLARE @BusinessEntityID int;
WHILE @i < 250
    BEGIN
        SELECT TransactionID FROM Production.TransactionHistory WHERE ProductID = 100001;
        UPDATE Production.TransactionHistory SET ReferenceOrderLineID = 1 WHERE TransactionID = 100001;
        SET @i = @i + 1;
        PRINT @i;
    END;



/***********************************************************************************
* Index Usage Stats
* Brent Ozar - http://www.brentozar.com/blitzindex/
*
* This is a great script that finds index usage and comes with a script to remove
* indexes that are not used.  This script was written by Brent Ozar and has been
* incorporated into his BlitzIndex script.
*
* Using this script we can see that our IX_TransactionHistory_ReferenceOrderID_ReferenceOrderLineID
* Index is being updated constantly but never read from, where as our IX_TransactionHistory_ProductID
* is read extensively and never written to.
***********************************************************************************/
SELECT o.name
     , indexname = i.name
     , i.index_id
     , reads = user_seeks + user_scans + user_lookups
     , writes = user_updates
     , rows = (
               SELECT SUM(p.rows)
               FROM sys.partitions p
               WHERE p.index_id = s.index_id
                 AND s.object_id = p.object_id)
     , reads_per_write = CASE WHEN s.user_updates < 1 
	                         THEN 100 
						ELSE 1.00 * (s.user_seeks + s.user_scans + s.user_lookups) / s.user_updates 
					END
     , drop_statement = 'DROP INDEX ' + QUOTENAME(i.name) + 
	                   ' ON ' + QUOTENAME(c.name) + '.' + QUOTENAME(OBJECT_NAME(s.object_id))
FROM sys.dm_db_index_usage_stats s
     INNER JOIN sys.indexes i ON i.index_id = s.index_id
                             AND s.object_id = i.object_id
     INNER JOIN sys.objects o ON s.object_id = o.object_id
     INNER JOIN sys.schemas c ON o.schema_id = c.schema_id
WHERE OBJECTPROPERTY(s.object_id , 'IsUserTable') = 1
  AND s.database_id = DB_ID()
  AND i.type_desc = 'nonclustered'
  AND i.is_primary_key = 0
  AND i.is_unique_constraint = 0
  AND (
       SELECT SUM(p.rows)
       FROM sys.partitions p
       WHERE p.index_id = s.index_id
         AND s.object_id = p.object_id) > 100000
ORDER BY reads , writes DESC;
GO



/***********************************************************************************
* CAUTION...
* Use caution with this because DMV's only store data from the last restart of
* the SQL Server services.
*
* Restart SQL Server Service...
***********************************************************************************/



/***********************************************************************************
* Index Usage Stats
* Brent Ozar - http://www.brentozar.com/blitzindex/
*
* After the SQL Server services restart we see that our history is gone
***********************************************************************************/

SELECT o.name
     , indexname = i.name
     , i.index_id
     , reads = user_seeks + user_scans + user_lookups
     , writes = user_updates
     , rows = (
               SELECT SUM(p.rows)
               FROM sys.partitions p
               WHERE p.index_id = s.index_id
                 AND s.object_id = p.object_id)
     , reads_per_write = CASE WHEN s.user_updates < 1 
	                         THEN 100 
						ELSE 1.00 * (s.user_seeks + s.user_scans + s.user_lookups) / s.user_updates 
					END
     , drop_statement = 'DROP INDEX ' + QUOTENAME(i.name) + 
	                   ' ON ' + QUOTENAME(c.name) + '.' + QUOTENAME(OBJECT_NAME(s.object_id))
FROM sys.dm_db_index_usage_stats s
     INNER JOIN sys.indexes i ON i.index_id = s.index_id
                             AND s.object_id = i.object_id
     INNER JOIN sys.objects o ON s.object_id = o.object_id
     INNER JOIN sys.schemas c ON o.schema_id = c.schema_id
WHERE OBJECTPROPERTY(s.object_id , 'IsUserTable') = 1
  AND s.database_id = DB_ID()
  AND i.type_desc = 'nonclustered'
  AND i.is_primary_key = 0
  AND i.is_unique_constraint = 0
  AND (
       SELECT SUM(p.rows)
       FROM sys.partitions p
       WHERE p.index_id = s.index_id
         AND s.object_id = p.object_id) > 100000
ORDER BY reads , writes DESC;
GO


/* END Non-Used Indexes DEMO */